/*
 * GPS Navigation http://launchpad.net/unav
 * Copyright (C) 2015-2016 Marcos Alvarez Costales https://launchpad.net/~costales
 * Copyright (C) 2015-2016 JkB https://launchpad.net/~joergberroth
 * Copyright (C) 2016 Nekhelesh Ramananthan http://launchpad.net/~nik90
 *
 * GPS Navigation is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * GPS Navigation is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

import QtQuick 2.4
import Ubuntu.Components 1.3
import Ubuntu.Components.Popups 1.3
import QtQuick.XmlListModel 2.0
import "js/utils.js" as QmlJs

Page {
    id: poiListPage

    property var fromPage
    property var lat
    property var lng
    property string poiType: ""
    property string clause: ""

    property double geoDistFactor: 1.0

    header: PageHeader {
        title: poiType
        flickable: resultsListView
        trailingActionBar.actions: Action {
            id: routeAction
            iconSource: "../nav/img/header/poimap.svg"
            text: i18n.tr("Show POIs on map")
            visible: sortedPoiModel.count !== 0
            onTriggered: {
                if (mainPageStack.center_onpos === 2)
                    mainPageStack.center_onpos = 1;
                mainPageStack.executeJavaScript("ui.markers_POI_set(" + JSON.stringify(sortedPoiModel.allPOI()) + ");")
                mainPageStack.pop(poiListPage.fromPage)
                mainPageStack.pop(poiListPage)
            }
        }
    }

    anchors.fill: parent

    Component.onCompleted: {
        resultsListView.visible = false
        if (poiListPage.lat !== "null" && poiListPage.lng !== "null") {
            poiXmlModel.search();
        } else {
            statusLabel.text = i18n.tr("Unknown current position")
            statusLabel.visible = true;
        }
    }

    Label {
        id: statusLabel
        anchors {
            centerIn: parent
            horizontalCenter: parent.horizontalCenter
            verticalCenter: parent.verticalCenter
        }
        width: parent.width - units.gu(4)
        visible: poiXmlModel.status === XmlListModel.Loading
        wrapMode: Text.WordWrap
        horizontalAlignment: Text.AlignHCenter
        text: i18n.tr("Something was wrong. Please, try again…")
    }

    // Indicator to show load activity
    ActivityIndicator {
        id: searchActivity
        anchors {
            bottom: statusLabel.top
            bottomMargin: units.gu (1)
            horizontalCenter: parent.horizontalCenter
        }
        running: poiXmlModel.status === XmlListModel.Loading
    }

    Slider {
        id: distSlider
        width: parent.width - units.gu(4)
        anchors {
            top: statusLabel.bottom
            topMargin: units.gu (4)
            horizontalCenter: parent.horizontalCenter
        }
        visible: false
        z: 500

        function formatValue(v) {
            return v.toFixed(0).toString() + (navApp.settings.unit === 0 ? " km" : " mi" )
        }
        minimumValue: 5.0
        maximumValue: 100.0
        value: geoDistFactor <= 25 ? 25.0 : geoDistFactor
        live: true
    }

    Button {
        id: distButton
        visible: false
        z: 500
        anchors {
            top: distSlider.bottom
            topMargin: units.gu (1)
            horizontalCenter: parent.horizontalCenter
        }
        iconName: "reload"
        onClicked: {
            poiListPage.geoDistFactor = distSlider.value
            poiXmlModel.search()
            distButton.visible = false;
            distSlider.visible = false;
        }
    }


    //OSMTouch (lp:osmtouch) POIModel:
    XmlListModel {
        id: poiXmlModel

        onStatusChanged: {

            if (status === XmlListModel.Error) {
                console.log(errorString())
                statusLabel.text = i18n.tr("Time out! Please try again");
                statusLabel.visible = true;
                resultsListView.visible = false;
                source = "";
            }
            if (status === XmlListModel.Ready && count === 0) {
                //TRANSLATORS: The Argument is the POI type. E.G. a pub, station....
                statusLabel.text = i18n.tr("Sorry, no %1 found nearby. Try again with another radius").arg(poiType);
                statusLabel.visible = true;
                distButton.visible = true;
                distSlider.visible = true;
                resultsListView.visible = false;
            }
            if (status === XmlListModel.Ready && count >> 0) {
                statusLabel.visible = false;
                sortedPoiModel.sortXmlList();
            }
            if (status === XmlListModel.Loading) { statusLabel.text =  i18n.tr("Searching…") }
        }

        readonly property string baseUrl: "http://nominatim.openstreetmap.org/search?format=xml&bounded=1&limit=50&email=costales.marcos@gmail.com&extratags=1"
        readonly property double geoDist: navApp.settings.unit === 0 ? 0.01 : 0.01 / 0.621371192
        // geographic distance ~1.1km / ~1.1mi
        // rough estimation only. Could be redefined.
        function search() {
            // Boxed area in which to search for PoI
            var bbox = ( Number(poiListPage.lng) - geoDist*geoDistFactor).toString() + ","
                     + ( Number(poiListPage.lat) - geoDist*geoDistFactor).toString() + ","
                     + ( Number(poiListPage.lng) + geoDist*geoDistFactor).toString() + ","
                     + ( Number(poiListPage.lat) + geoDist*geoDistFactor).toString();
            source = (baseUrl + "&q=" + clause + "&viewbox=" + bbox);
        }
        function clear() {
            source: "";
        }

        source: ""
        query: "/searchresults/place"

        XmlRole { name: "osm_id"; query: "@place_id/string()"; }
        XmlRole { name: "name"; query: "@display_name/string()"; }
        XmlRole { name: "description"; query: "extratags/tag[@key='description']/@value/string()"; }
        XmlRole { name: "phone"; query: "extratags/tag[@key='phone']/@value/string()"; }
        XmlRole { name: "website"; query: "extratags/tag[@key='website']/@value/string()"; }
        // XmlRole { name: "cuisine"; query: "extratags/tag[@key='cuisine']/@value/string()"; }
        XmlRole { name: "wheelchair"; query: "extratags/tag[@key='wheelchair']/@value/string()"; }
        XmlRole { name: "openinghours"; query: "extratags/tag[@key='opening_hours']/@value/string()"; }
        XmlRole { name: "lat"; query: "@lat/string()"; }
        XmlRole { name: "lng"; query: "@lon/string()"; }
    }

    ListModel {
        id: sortedPoiModel

        function allPOI() {
            var allPOI = [];
            for (var i = 0; i < sortedPoiModel.count; i++) {
                allPOI.push({
                    title: sortedPoiModel.get(i).name,
                    lat: parseFloat(sortedPoiModel.get(i).lat),
                    lng: parseFloat(sortedPoiModel.get(i).lng),
                    website: sortedPoiModel.get(i).website,
                    phone: sortedPoiModel.get(i).phone
                });
            }
            return allPOI;
        }

        function sortXmlList (){
            var item
            for (var i = 0; i < poiXmlModel.count; i++) {
               item  = {"osmid":        poiXmlModel.get(i).osm_id,
                        "name":             poiXmlModel.get(i).name.split(',')[0],
                        "description":  poiXmlModel.get(i).description,
                        "phone":        poiXmlModel.get(i).phone,
                        "website":        poiXmlModel.get(i).website,
                        // "cuisine":    poiXmlModel.get(i).cuisine,
                        "wheelchair":    poiXmlModel.get(i).wheelchair,
                        "openinghours":    poiXmlModel.get(i).openinghours,
                        "lat":            poiXmlModel.get(i).lat,
                        "lng":            poiXmlModel.get(i).lng,
                        "distance":        QmlJs.calcPoiDistance(
                            poiListPage.lat,
                            poiListPage.lng,
                            poiXmlModel.get(i).lat,
                            poiXmlModel.get(i).lng,
                            10
                        ),
                    }
                if (i === 0) {
                    sortedPoiModel.append(item)
                        } else { // sort model by distance
                    var j = 0;
                    while (j <= sortedPoiModel.count) {
                        if (j === sortedPoiModel.count) {
                            sortedPoiModel.append(item)
                            break;
                        } else if (item.distance < sortedPoiModel.get(j).distance){
                           sortedPoiModel.insert(j,item)
                           break;
                        } else {
                            j++;
                        }
                    }
                }
                poiXmlModel.clear();
                resultsListView.visible = true;
            }
        }
    }

    ListView {
        id: resultsListView

        model: sortedPoiModel
        anchors.fill: parent
        visible: false

        delegate: ListItem {
            height: poiListItemLayout.height + divider.height
            trailingActions:  ListItemActions {
                actions: [
                    Action {
                        iconName: "send"
                        onTriggered: {
                            mainPageStack.pop(poiListPage.fromPage)
                            mainPageStack.pop(poiListPage)
                            mainPageStack.center_onpos = 2;
                            mainPageStack.routeState = 'yes'
                            mainPageStack.executeJavaScript("calc2coord("+ model.lat + "," + model.lng + ");");
                        }
                    },
                    Action {
                        iconName: "non-starred"
                        onTriggered: {
                            mainPageStack.pop(poiListPage.fromPage)
                            mainPageStack.pop(poiListPage)
                            mainPageStack.push(Qt.resolvedUrl("FavoritesPage.qml"), {isAddedFromPopup: true, lat: model.lat, lng: model.lng, favName: model.name})
                        }
                    },
                    Action {
                        iconName: "share"
                        onTriggered: {
                            PopupUtils.open(Qt.resolvedUrl("./Share.qml"), navApp, {"lat": model.lat, "lon": model.lng})
                        }
                    },
                    Action {
                        iconName: "external-link"
                        visible: (model.website !== "" && model.website.substring(0, 4) === "http")
                        onTriggered: {
                            Qt.openUrlExternally(website)
                        }
                    },
                    Action {
                        iconName: "call-start"
                        visible: model.phone !== ""
                        onTriggered: {
                            Qt.openUrlExternally("tel:///"+ phone)
                        }
                    }
                ]
            }

            onClicked: {
                mainPageStack.pop(poiListPage.fromPage)
                mainPageStack.pop(poiListPage);
                mainPageStack.executeJavaScript("ui.markers_POI_set([{title: \"" + model.name + "\", lat: " + model.lat + ", lng: " + model.lng + ", website: \"" + model.website + "\", phone: \"" + model.phone + "\"}]);");
            }

            ListItemLayout {
                id: poiListItemLayout

                function getOpeningHours(openinghours) {
                    openinghours.replace("Mo", Qt.locale().dayName(1, Locale.ShortFormat)).
                    replace("Tu", Qt.locale().dayName(2, Locale.ShortFormat)).
                    replace("We", Qt.locale().dayName(3, Locale.ShortFormat)).
                    replace("Th", Qt.locale().dayName(4, Locale.ShortFormat)).
                    replace("Fr", Qt.locale().dayName(5, Locale.ShortFormat)).
                    replace("Sa", Qt.locale().dayName(6, Locale.ShortFormat)).
                    replace("Su", Qt.locale().dayName(0, Locale.ShortFormat)).
                    //TRANSLATORS: abbreviation for Public Holiday
                    replace("PH", i18n.tr("PH")).
                    //TRANSLATORS: "closed"
                    replace("off", i18n.tr("off"))

                    return openinghours
                }

                title.text: model.name !== "" ? model.name : poiListPage.poiType
                subtitle.text: model.openinghours !== "" ? "%1 | %2".arg(getOpeningHours(model.openinghours)).arg(QmlJs.formatDistance(model.distance, navApp.settings.unit))
                                                         : QmlJs.formatDistance(model.distance, navApp.settings.unit)
                subtitle.maximumLineCount: 2
                subtitle.wrapMode: Text.WordWrap
                summary.text: model.description
                summary.maximumLineCount: 3

                Icon {
                    id: acessibilityIcon
                    name: "preferences-desktop-accessibility-symbolic"
                    visible: model.wheelchair === "yes" || model.wheelchair === "limited" // is limited enough as criteria?
                    width: parent.height / 2
                    SlotsLayout.position: SlotsLayout.Last
                }
            }
        }
    }
}
